@@ -10,8 +10,30 @@ if __name__ == '__main__':
   from zebrazoom.code.vars import getGlobalVariables
   globalVariables = getGlobalVariables()
 
+  requiredFolders = (paths.getDefaultZZoutputFolder(), paths.getConfigurationFolder(),
+                     os.path.join(paths.getDataAnalysisFolder(), 'data'),
+                     os.path.join(paths.getDataAnalysisFolder(), 'experimentOrganizationExcel'),
+                     os.path.join(paths.getDataAnalysisFolder(), 'resultsClustering'),
+                     os.path.join(paths.getDataAnalysisFolder(), 'resultsKinematic'))
+  errors = []
+  for folder in requiredFolders:
+    try:
+      os.makedirs(folder, exist_ok=True)
+    except OSError:
+      errors.append(folder)
+  if errors:
+    errorMessage = "Some of the folders required by ZebraZoom are missing and could not be created:\n" \
+                   "%s\n\nZebraZoom cannot work without these folders, please make sure you have adequate " \
+                   "write permissions or try an alternative installation method." % '\n'.join(errors)
+
   if len(sys.argv) == 1:
 
+    if errors:
+      from PyQt5.QtWidgets import QMessageBox
+      from zebrazoom.GUIAllPy import PlainApplication
+      app = PlainApplication(sys.argv)
+      QMessageBox.critical(None, "Required folders missing", errorMessage)
+      sys.exit(1)
     from zebrazoom.GUIAllPy import ZebraZoomApp
     print("The data produced by ZebraZoom can be found in the folder: " + paths.getDefaultZZoutputFolder())
     app = ZebraZoomApp(sys.argv)
@@ -19,6 +41,10 @@ if __name__ == '__main__':
 
   else:
 
+    if errors:
+      print(errorMessage)
+      sys.exit(1)
+
     if sys.argv[1] == "selectZZoutput" or sys.argv[1] == "--exit":
       from zebrazoom.GUIAllPy import ZebraZoomApp
       app = ZebraZoomApp(sys.argv)
@@ -215,6 +241,9 @@ if __name__ == '__main__':
         from zebrazoom.otherScripts.launchReapplyClustering import launchReapplyClustering
         launchReapplyClustering()
 
+    elif sys.argv[1] == 'createSmallValidationVideosForFlagged':
+      from zebrazoom.code.createValidationVideo import createSmallValidationVideosForFlagged
+      createSmallValidationVideosForFlagged(sys.argv[2].rstrip(r'\/'), int(sys.argv[3]))
     elif sys.argv[1] == "--exit":
       from PyQt5.QtCore import QTimer
       QTimer.singleShot(0, app.window.close)
