@@ -1,4 +1,5 @@
 import os
+import scipy
 import scipy.io
 import pandas as pd
 import json
@@ -46,7 +47,9 @@ def createDataFrame(dataframeOptions, excelFileDataFrame="", forcePandasDfRecrea
   saveAllBoutsSuperStructuresInMatlabFormat = int(dataframeOptions['saveAllBoutsSuperStructuresInMatlabFormat']) if 'saveAllBoutsSuperStructuresInMatlabFormat' in dataframeOptions else 0
   
   getTailAngleSignMultNormalized = int(dataframeOptions['getTailAngleSignMultNormalized']) if 'getTailAngleSignMultNormalized' in dataframeOptions else 0
-  
+
+  gaussianFitOutlierRemoval = dataframeOptions.get('gaussianFitOutlierRemoval', False)
+
   # If nbFramesTakenIntoAccount was not specified, finds an appropriate value for it
   nbFramesTakenIntoAccount          = dataframeOptions['nbFramesTakenIntoAccount']
   if len(pathToExcelFile):
@@ -94,9 +97,9 @@ def createDataFrame(dataframeOptions, excelFileDataFrame="", forcePandasDfRecrea
   basicInformation = ['Trial_ID', 'Well_ID', 'NumBout', 'BoutStart', 'BoutEnd', 'Condition', 'Genotype', 'videoDuration']
   # Global parameters
   if tailAngleKinematicParameterCalculation:
-    globParam  = ['BoutDuration', 'Bout Distance (mm)', 'Bout Speed (mm/s)', 'Max TBF (Hz)', 'Mean TBF (Hz)', 'medianOfInstantaneousTBF', 'Max absolute TBA (deg.)', 'maxBendAmplitudeSigned', 'Mean absolute TBA (deg.)', 'Median absolute TBA (deg.)', 'medianBendAmplitudeSigned', 'Number of Oscillations', 'meanTBF', 'maxTailAngleAmplitude', 'Absolute Yaw (deg)', 'TBA#1 timing (deg)', 'TBA#1 Amplitude (deg)', 'firstBendAmplitudeSigned', 'IBI (s)', 'xmean', 'ymean', 'binaryClass25degMaxTailAngle', 'tailAngleIntegralSigned', 'BoutFrameNumberStart', 'tailAngleSymmetry', 'secondBendAmpDividedByFirst', 'tailAngleIntegral', 'maxInstantaneousSpeed']
+    globParam  = ['Bout Duration (s)', 'Bout Distance (mm)', 'Bout Speed (mm/s)', 'Max TBF (Hz)', 'Mean TBF (Hz)', 'medianOfInstantaneousTBF', 'Max absolute TBA (deg.)', 'maxBendAmplitudeSigned', 'Mean absolute TBA (deg.)', 'Median absolute TBA (deg.)', 'medianBendAmplitudeSigned', 'Number of Oscillations', 'meanTBF', 'maxTailAngleAmplitude', 'Absolute Yaw (deg)', 'Signed Yaw (deg)', 'TBA#1 timing (deg)', 'TBA#1 Amplitude (deg)', 'firstBendAmplitudeSigned', 'IBI (s)', 'xmean', 'ymean', 'binaryClass25degMaxTailAngle', 'tailAngleIntegralSigned', 'BoutFrameNumberStart', 'tailAngleSymmetry', 'secondBendAmpDividedByFirst', 'tailAngleIntegral', 'maxInstantaneousSpeed']
   else:
-    globParam  = ['BoutDuration', 'Bout Distance (mm)', 'Bout Speed (mm/s)']
+    globParam  = ['Bout Duration (s)', 'Bout Distance (mm)', 'Bout Speed (mm/s)']
   
   if type(addToGlobalParameters) == list:
     globParam = globParam + addToGlobalParameters  
@@ -141,7 +144,7 @@ def createDataFrame(dataframeOptions, excelFileDataFrame="", forcePandasDfRecrea
   conditions = []
   # This is for the potential reload from previously calculated parameters (stored in the pkl file)
   if keepSpeedDistDurWhenLowNbBends == 1:
-    onlyKeepTheseColumns = basicInformation + ['BoutDuration', 'Bout Distance (mm)', 'Bout Speed (mm/s)', 'IBI (s)']
+    onlyKeepTheseColumns = basicInformation + ['Bout Duration (s)', 'Bout Distance (mm)', 'Bout Speed (mm/s)', 'IBI (s)']
   else:
     onlyKeepTheseColumns = basicInformation
   removeColumnsWhenAppropriate = [col for col in dfCols if not(col in onlyKeepTheseColumns)]
@@ -175,30 +178,44 @@ def createDataFrame(dataframeOptions, excelFileDataFrame="", forcePandasDfRecrea
     else:
       print("reloading previously calculated parameters")
       dfReloadedVid = pd.read_pickle(os.path.join(path, trial_id + '.pkl'))
-      
-      nbFramesTakenIntoAccountReloaded = max([np.sum(['instaTBF' in param for param in dfReloadedVid.columns.tolist()]), np.sum(['tailAnglesRecalculated' in param for param in dfReloadedVid.columns.tolist()]), np.sum(['instaSpeed' in param for param in dfReloadedVid.columns.tolist()])])
-      if nbFramesTakenIntoAccountReloaded < nbFramesTakenIntoAccount:
-        raise ValueError("nbFramesTakenIntoAccount was too low when pre-generating the pkl file of a video:" + str(np.sum(['instaTBF' in param for param in dfReloadedVid.columns.tolist()])) + " , " + str(np.sum(['tailAnglesRecalculated' in param for param in dfReloadedVid.columns.tolist()])) + " , " + str(np.sum(['instaSpeed' in param for param in dfReloadedVid.columns.tolist()])) + " ; nbFramesTakenIntoAccount :" + str(nbFramesTakenIntoAccount))
-        
-      for idx, cond in enumerate(condition):
-        indForWellId = (dfReloadedVid['Well_ID'] == idx)
-        if include[idx]:
-          dfReloadedVid.loc[indForWellId, 'Condition'] = cond
-          dfReloadedVid.loc[indForWellId, 'Genotype']  = genotype[idx]
-          if minNbBendForBoutDetect > 0:
-            ind           = (dfReloadedVid['Number of Oscillations'] < minNbBendForBoutDetect/2)
-            dfReloadedVid.loc[ind, removeColumnsWhenAppropriate] = float('NaN')
-          if not(genotype[idx] in genotypes):
-            genotypes.append(genotype[idx])
-          if not(condition[idx] in conditions):
-            conditions.append(condition[idx])
+      if 'BoutDuration' in dfReloadedVid:  # pickle file contains old parameter names, recreate it
+        forcePandasDfRecreation = True
+        if len(supstructOverwrite):
+          supstruct = supstructOverwrite
         else:
-          dfReloadedVid = dfReloadedVid.drop([idx2 for idx2, belongsToWell in enumerate(indForWellId) if belongsToWell])
-          if 'level_0' in dfReloadedVid.columns:
-            dfReloadedVid = dfReloadedVid.drop(['level_0'], axis=1)
-          dfReloadedVid = dfReloadedVid.reset_index()
+          with open(os.path.join(path, 'results_' + trial_id + '.txt')) as f:
+            supstruct = json.load(f)
+        firstFrame = supstruct["firstFrame"]
+        lastFrame  = supstruct["lastFrame"]
+        with open(os.path.join(path, 'parametersUsedForCalculation.json'), 'w') as outfile:
+          print('frameStepForDistanceCalculation', frameStepForDistanceCalculation)
+          print('videoFPS', excelFile.loc[videoId, 'fq'])
+          print('videoPixelSize', excelFile.loc[videoId, 'pixelsize'])
+          json.dump({'frameStepForDistanceCalculation': int(frameStepForDistanceCalculation), 'videoFPS': float(excelFile.loc[videoId, 'fq']), 'videoPixelSize': float(excelFile.loc[videoId, 'pixelsize'])}, outfile)
+      else:
+        nbFramesTakenIntoAccountReloaded = max([np.sum(['instaTBF' in param for param in dfReloadedVid.columns.tolist()]), np.sum(['tailAnglesRecalculated' in param for param in dfReloadedVid.columns.tolist()]), np.sum(['instaSpeed' in param for param in dfReloadedVid.columns.tolist()])])
+        if nbFramesTakenIntoAccountReloaded < nbFramesTakenIntoAccount:
+          raise ValueError("nbFramesTakenIntoAccount was too low when pre-generating the pkl file of a video:" + str(np.sum(['instaTBF' in param for param in dfReloadedVid.columns.tolist()])) + " , " + str(np.sum(['tailAnglesRecalculated' in param for param in dfReloadedVid.columns.tolist()])) + " , " + str(np.sum(['instaSpeed' in param for param in dfReloadedVid.columns.tolist()])) + " ; nbFramesTakenIntoAccount :" + str(nbFramesTakenIntoAccount))
           
-      dfParam = pd.concat([dfParam, dfReloadedVid])
+        for idx, cond in enumerate(condition):
+          indForWellId = (dfReloadedVid['Well_ID'] == idx)
+          if include[idx]:
+            dfReloadedVid.loc[indForWellId, 'Condition'] = cond
+            dfReloadedVid.loc[indForWellId, 'Genotype']  = genotype[idx]
+            if minNbBendForBoutDetect > 0:
+              ind           = (dfReloadedVid['Number of Oscillations'] < minNbBendForBoutDetect/2)
+              dfReloadedVid.loc[ind, removeColumnsWhenAppropriate] = float('NaN')
+            if not(genotype[idx] in genotypes):
+              genotypes.append(genotype[idx])
+            if not(condition[idx] in conditions):
+              conditions.append(condition[idx])
+          else:
+            dfReloadedVid = dfReloadedVid.drop([idx2 for idx2, belongsToWell in enumerate(indForWellId) if belongsToWell])
+            if 'level_0' in dfReloadedVid.columns:
+              dfReloadedVid = dfReloadedVid.drop(['level_0'], axis=1)
+            dfReloadedVid = dfReloadedVid.reset_index()
+
+        dfParam = pd.concat([dfParam, dfReloadedVid])
     
     # Going through each well of the video
     for Well_ID, Cond in enumerate(condition):
@@ -296,9 +313,9 @@ def createDataFrame(dataframeOptions, excelFileDataFrame="", forcePandasDfRecrea
                   # Calculating the global kinematic parameters and more and stores them the dataframe
                   
                   previousBoutEnd = supstruct["wellPoissMouv"][Well_ID][fishId][NumBout-1]["BoutEnd"] if NumBout > 0 else 0
-                  listOfGlobalParameters = getGlobalParameters(dataForBout, fq, pixelsize, frameStepForDistanceCalculation, previousBoutEnd, ['BoutDuration', 'Bout Distance (mm)', 'Bout Speed (mm/s)', 'IBI (s)'] + addToGlobalParameters, firstFrame, lastFrame, minimumFrameToFrameDistanceToBeConsideredAsMoving)
+                  listOfGlobalParameters = getGlobalParameters(dataForBout, fq, pixelsize, frameStepForDistanceCalculation, previousBoutEnd, ['Bout Duration (s)', 'Bout Distance (mm)', 'Bout Speed (mm/s)', 'IBI (s)'] + addToGlobalParameters, firstFrame, lastFrame, minimumFrameToFrameDistanceToBeConsideredAsMoving)
                   
-                  toPutInDataFrameColumn = toPutInDataFrameColumn + ['BoutDuration', 'Bout Distance (mm)', 'Bout Speed (mm/s)', 'IBI (s)'] + addToGlobalParameters
+                  toPutInDataFrameColumn = toPutInDataFrameColumn + ['Bout Duration (s)', 'Bout Distance (mm)', 'Bout Speed (mm/s)', 'IBI (s)'] + addToGlobalParameters
                   toPutInDataFrame       = toPutInDataFrame       + listOfGlobalParameters
                   
                 # Adding bout parameters to the dataframe created for the current well
@@ -313,7 +330,12 @@ def createDataFrame(dataframeOptions, excelFileDataFrame="", forcePandasDfRecrea
   if 'level_0' in dfParam.columns:
     dfParam = dfParam.drop(['level_0'], axis=1)
   dfParam = dfParam.reset_index()
-  
+
+  # Gaussian fit outlier removal
+  if gaussianFitOutlierRemoval:
+    columnsToCheck = [col for col in ('Bout Duration (s)', 'Bout Distance (mm)', 'Number of Oscillations', 'Max absolute TBA (deg.)', 'Absolute Yaw (deg)') if col in dfParam.columns]
+    dfParam.loc[(np.abs(scipy.stats.zscore(dfParam[columnsToCheck].astype(float), nan_policy='omit')) > 3).any(axis=1), ~dfParam.columns.isin(basicInformation)] = np.nan
+
   # Saving dataframe for the whole set of videos as a pickle file
   outfile = open(os.path.join(resFolder, nameOfFile + '.pkl'), 'wb')
   pickle.dump(dfParam,outfile)
