@@ -104,9 +104,12 @@ class ZzVideoReading():
       else:
         buffer = np.frombuffer(bytes, dtype=np.uint8)
       
-      nparr2 = buffer.reshape(self.height, self.width)
       
-      nparr2 = cv2.cvtColor(nparr2, cv2.COLOR_GRAY2RGB)
+      if len(buffer) == 3*self.height*self.width:
+        nparr2 = buffer.reshape(self.height, self.width, 3)
+      else:
+        nparr2 = buffer.reshape(self.height, self.width)
+        nparr2 = cv2.cvtColor(nparr2, cv2.COLOR_GRAY2RGB)
       
       self.lastFrameRead = self.lastFrameRead + 1
       
@@ -129,11 +132,11 @@ class ZzVideoReading():
       
       elif numImage >= self.num_images:
         
-        self.lastFrameRead = self.num_images
+        self.lastFrameRead = int(self.num_images)
         
       elif numImage > self.lastFrameRead:
         
-        justToMove = self.f.read(24 * (numImage - self.lastFrameRead - 1))
+        justToMove = self.f.read(24 * (int(numImage) - int(self.lastFrameRead) - 1))
         self.lastFrameRead = numImage - 1  
         
       elif numImage < self.lastFrameRead:
@@ -141,7 +144,7 @@ class ZzVideoReading():
         self.f.close()
         self.f = open(self.sqb_path, 'rb')
         
-        justToMove = self.f.read(24 * numImage)
+        justToMove = self.f.read(24 * int(numImage))
         self.lastFrameRead = numImage - 1
 
 
